


####  CD4/CD8 analyses

rm(list=ls())
setwd("~/Documents/data-files/jessie-cd4-cd8")

hgb.d<-read.csv("cd4_cd8_base_hgb_28jul2015.csv") 
ncds.d<-read.csv("cd4cd8rat_ncds_04jun2015.csv")
demo.d<-read.csv("cd4cd8rat_pts_04jun2015.csv")
labs.d<-read.csv("cd4cd8ratio_labs_04jun2015.csv")
extra.d<-read.csv("extra_variables_26oct2015.csv")
unk.vl.d<-read.csv("pts_unk_ud_vl_duration_22sep2015.csv")

d<-demo.d
d$cd4<-d$BASELINE_CD4_COUNT
d$cd8<-d$BASELINE_CD8_COUNT
d$cd4.cd8.ratio<-with(d, cd4/cd8)
d$y<-d$cd4.cd8.ratio

d<-d[!is.na(d$y),]
d<-d[!is.na(d$BASELINE_VL),]
d<-d[d$BASELINE_VL<400,]

d$race<-with(d, ifelse(raceeth=="African American", "African American",
                       ifelse(raceeth=="Caucasian", "Caucasian",
                              ifelse(raceeth=="Hispanic", "Hispanic", "Other"))))
d$female<-ifelse(d$sex=="F",1,0)
d$age<-d$AGE_AT_T0
d$route<-with(d, ifelse(RISK_FACTOR_1=="Other"|RISK_FACTOR_1=="Unknown","Other/Unknown",
                        as.character(RISK_FACTOR_1)))
d$hcv<-ifelse(is.na(d$HCV_DX_PRIOR_T0),0,d$HCV_DX_PRIOR_T0)
d$hbv<-ifelse(is.na(d$HBV_DX_PRIOR_T0),0,d$HBV_DX_PRIOR_T0)
d$year<-d$YEAR_OF_T0

summary(d$y)
sum(d$y<=.05)
sum(d$y<=.1)  ###  6/407 in Serrano-Villa et al., 2014, PLOS ONE 
sum(d$y<=.2)  ###  28/407 
sum(d$y>=1.5)  ### 19/407
sum(d$y>=2)    ###  2/407
sum(d$y>=3)

d$y1<-with(d,ifelse(y<=0.1,.1,
                    ifelse(y>=2,2,y)))

d$y2<-with(d,ifelse(y<=0.2,.2,
                    ifelse(y>=1.5,1.5,y)))


library(rms)
dd <- datadist(d)
options(datadist='dd')
source("~/Dropbox/chun-stuff/donglin/application-analysis/qi-function-modified.R")

mod <- orm(y ~ female + race + rcs(age, 4) + route + hcv + hbv + year, 
           data=d, x=TRUE, y=TRUE)

mod1 <- orm(y1 ~ female + race + rcs(age, 4) + route + hcv + hbv + year, 
            data=d, x=TRUE, y=TRUE)

mod2 <- orm(y2 ~ female + race + rcs(age, 4) + route + hcv + hbv + year, 
            data=d, x=TRUE, y=TRUE)

anova(mod)
anova(mod1)
anova(mod2)

summary(mod)
summary(mod1)
summary(mod2)

cbind(summary(mod)[2*(1:11),4],summary(mod)[2*(1:11),6],summary(mod)[2*(1:11),7])
cbind(summary(mod1)[2*(1:11),4],summary(mod1)[2*(1:11),6],summary(mod1)[2*(1:11),7])
cbind(summary(mod2)[2*(1:11),4],summary(mod2)[2*(1:11),6],summary(mod2)[2*(1:11),7])


age_knots <- rcspline.eval(d$age, nk=4, knots.only=TRUE)
age_splines_med <- rcspline.eval(median(d$age), inclx=TRUE, knots=c(age_knots))
new.data.med<-c(0,1,0,0,age_splines_med[1,1],age_splines_med[1,2],age_splines_med[1,3],
                0, 1, 0, 0, 0, median(d$year))
age_splines_seq <- rcspline.eval(c(18:75), inclx=TRUE, knots=c(age_knots))

new.data.age<-matrix(NA,dim(age_splines_seq)[1],13)
for (i in 1:dim(age_splines_seq)[1]) {
  new.data.age[i,1:4]<-new.data.med[1:4]
  new.data.age[i,5:7]<-age_splines_seq[i,]
  new.data.age[i,8:13]<-new.data.med[8:13]
}

colnames(new.data.age)<-names(tail(coef(mod),13))

means<-data.frame(mean.orm(mod, new.data=new.data.age, se=TRUE))
medians<-quantile.orm(mod, new.data=new.data.age, probs=0.5, se=TRUE)
cdfs<-cdf.orm(mod, new.data=new.data.age, at=1, se=TRUE)

means1<-data.frame(mean.orm(mod1, new.data=new.data.age, se=TRUE))
medians1<-quantile.orm(mod1, new.data=new.data.age, probs=0.5, se=TRUE)
cdfs1<-cdf.orm(mod1, new.data=new.data.age, at=1, se=TRUE)

means2<-data.frame(mean.orm(mod2, new.data=new.data.age, se=TRUE))
medians2<-quantile.orm(mod2, new.data=new.data.age, probs=0.5, se=TRUE)
cdfs2<-cdf.orm(mod2, new.data=new.data.age, at=1, se=TRUE)


#### Medians are not discernably different between the models

#plot(rep(age_splines_seq[,1],3),c(medians2$quantile,medians2$lb,medians2$ub),type="n",
#     xlab="age (years)", ylab="median CD4:CD8 ratio")
plot(c(18,75),c(.25,1.1),type="n",xlab="age (years)", ylab="median CD4:CD8 ratio")
lines(age_splines_seq[,1],medians$quantile, lwd=2)
lines(age_splines_seq[,1],medians$lb, col=gray(.5))
lines(age_splines_seq[,1],medians$ub, col=gray(.5))
#lines(age_splines_seq[,1],medians2$quantile)
#lines(age_splines_seq[,1],medians2$lb, col=gray(.5))
#lines(age_splines_seq[,1],medians2$ub, col=gray(.5))



#### Means are different

plot(rep(age_splines_seq[,1],3),c(means2$est,means2$lb,means2$ub),type="n",
     xlab="age", ylab="mean CD4:CD8 ratio")
lines(age_splines_seq[,1],means$est, lwd=2)
lines(age_splines_seq[,1],means$lb, col=gray(.5))
lines(age_splines_seq[,1],means$ub, col=gray(.5))
lines(age_splines_seq[,1],means2$est,col=2,lwd=2)
lines(age_splines_seq[,1],means2$lb, col=2)
lines(age_splines_seq[,1],means2$ub, col=2)


####  Really, the uncensored is the only one that is reasonable
#plot(rep(age_splines_seq[,1],3),c(means$est,means$lb,means$ub),type="n",
#     xlab="age (years)", ylab="mean CD4:CD8 ratio")
plot(c(18,75),c(.25,1.1),type="n",xlab="age (years)", ylab="mean CD4:CD8 ratio")
lines(age_splines_seq[,1],means$est, lwd=2)
lines(age_splines_seq[,1],means$lb, col=gray(.5))
lines(age_splines_seq[,1],means$ub, col=gray(.5))


plot(c(18,75),c(0,.6),type="n",xlab="age (years)", ylab="Pr(CD4:CD8 ratio >1)")
lines(age_splines_seq[,1],1-cdfs$est, lwd=2)
lines(age_splines_seq[,1],1-cdfs$lb, col=gray(.5))
lines(age_splines_seq[,1],1-cdfs$ub, col=gray(.5))
lines(age_splines_seq[,1],1-cdfs2$est, lwd=2, col=2)
lines(age_splines_seq[,1],1-cdfs2$lb, col=2)
lines(age_splines_seq[,1],1-cdfs2$ub, col=2)


#### Plot for manuscript

pdf("~/Dropbox/chun-stuff/donglin/application-analysis/cd4-cd8-plots.pdf")
par(mfrow=c(2,2),mar=c(4.1,4,2,1))
hist(d$y, xlab="CD4:CD8 ratio",main="a")

plot(c(18,75),c(.25,1.1),type="n",xlab="age (years)", ylab="median CD4:CD8 ratio")
title("b")
lines(age_splines_seq[,1],medians$quantile, lwd=2)
lines(age_splines_seq[,1],medians$lb, col=gray(.5))
lines(age_splines_seq[,1],medians$ub, col=gray(.5))

plot(c(18,75),c(.25,1.1),type="n",xlab="age (years)", ylab="mean CD4:CD8 ratio")
title("c")
lines(age_splines_seq[,1],means$est, lwd=2)
lines(age_splines_seq[,1],means$lb, col=gray(.5))
lines(age_splines_seq[,1],means$ub, col=gray(.5))

plot(c(18,75),c(0,.6),type="n",xlab="age (years)", ylab="Pr(CD4:CD8 ratio >1)")
title("d")
lines(age_splines_seq[,1],1-cdfs$est, lwd=2)
lines(age_splines_seq[,1],1-cdfs$lb, col=gray(.5))
lines(age_splines_seq[,1],1-cdfs$ub, col=gray(.5))

dev.off()



